@@ -185,6 +185,12 @@ pub struct Banner {
 
     /// represents Configuration.limit_bars
     limit_bars: BannerEntry,
+
+    /// represents Configuration.unique
+    unique: BannerEntry,
+
+    /// represents Configuration.response_size_limit
+    response_size_limit: BannerEntry,
 }
 
 /// implementation of Banner
@@ -429,6 +435,14 @@ impl Banner {
         let collect_words =
             BannerEntry::new("🤑", "Collect Words", &config.collect_words.to_string());
 
+        let unique = BannerEntry::new("🎲", "Unique Responses", &config.unique.to_string());
+
+        let response_size_limit = BannerEntry::new(
+            "📏",
+            "Response Size Limit",
+            &format!("{} bytes", config.response_size_limit),
+        );
+
         Self {
             targets,
             status_codes,
@@ -480,6 +494,8 @@ impl Banner {
             scan_dir_listings,
             protocol,
             limit_bars,
+            unique,
+            response_size_limit,
             version: VERSION.to_string(),
             update_status: UpdateStatus::Unknown,
         }
@@ -521,7 +537,7 @@ by Ben "epi" Risher {}                 ver: {}"#,
     ///
     /// ex: v1.1.0
     pub async fn check_for_updates(&mut self, url: &str, handles: Arc<Handles>) -> Result<()> {
-        log::trace!("enter: needs_update({}, {:?})", url, handles);
+        log::trace!("enter: needs_update({url}, {handles:?})");
 
         let api_url = parse_url_with_raw_path(url)?;
 
@@ -560,7 +576,7 @@ by Ben "epi" Risher {}                 ver: {}"#,
         let latest_version = match json_response["tag_name"].as_str() {
             Some(tag) => tag.trim_start_matches('v'),
             None => {
-                bail!("JSON has no tag_name: {}", json_response);
+                bail!("JSON has no tag_name: {json_response}");
             }
         };
 
@@ -778,6 +794,14 @@ by Ben "epi" Risher {}                 ver: {}"#,
             writeln!(&mut writer, "{}", self.time_limit)?;
         }
 
+        if config.unique {
+            writeln!(&mut writer, "{}", self.unique)?;
+        }
+
+        if config.response_size_limit != 4194304 {
+            writeln!(&mut writer, "{}", self.response_size_limit)?;
+        }
+
         if matches!(self.update_status, UpdateStatus::OutOfDate) {
             let update = BannerEntry::new(
                 "🎉",
