@@ -1,15 +1,17 @@
 use super::utils::{
-    depth, extract_links, ignored_extensions, methods, report_and_exit, save_state,
-    serialized_type, status_codes, threads, timeout, user_agent, wordlist, OutputLevel,
-    RequesterPolicy,
+    backup_extensions, depth, determine_requester_policy, extract_links, ignored_extensions,
+    methods, parse_request_file, report_and_exit, request_protocol, response_size_limit,
+    save_state, serialized_type, split_header, split_query, status_codes, threads, timeout,
+    user_agent, wordlist, OutputLevel, RequesterPolicy,
 };
+
 use crate::config::determine_output_level;
-use crate::config::utils::determine_requester_policy;
+use crate::config::utils::{preconfig_log, ContentType};
 use crate::{
     client, parser,
     scan_manager::resume_scan,
     traits::FeroxSerialize,
-    utils::{fmt_err, parse_url_with_raw_path},
+    utils::{fmt_err, module_colorizer, parse_url_with_raw_path, status_colorizer},
     DEFAULT_CONFIG_NAME,
 };
 use anyhow::{anyhow, Context, Result};
@@ -17,12 +19,14 @@ use clap::{parser::ValueSource, ArgMatches};
 use regex::Regex;
 use reqwest::{Client, Method, StatusCode, Url};
 use serde::{Deserialize, Serialize};
+use std::str::FromStr;
 use std::{
     collections::HashMap,
     env::{current_dir, current_exe},
     fs::read_to_string,
-    path::PathBuf,
+    path::{Path, PathBuf},
 };
+use url::form_urlencoded;
 
 /// macro helper to abstract away repetitive configuration updates
 macro_rules! update_config_if_present {
@@ -318,6 +322,9 @@ pub struct Configuration {
     #[serde(default)]
     pub collect_backups: bool,
 
+    #[serde(default = "backup_extensions")]
+    pub backup_extensions: Vec<String>,
+
     /// Automatically discover important words from within responses and add them to the wordlist
     #[serde(default)]
     pub collect_words: bool,
@@ -329,6 +336,30 @@ pub struct Configuration {
     /// Auto update app feature
     #[serde(skip)]
     pub update_app: bool,
+
+    /// whether to recurse into directory listings or not
+    #[serde(default)]
+    pub scan_dir_listings: bool,
+
+    /// path to a raw request file generated by burp or similar
+    #[serde(skip)]
+    pub request_file: String,
+
+    /// default request protocol
+    #[serde(default = "request_protocol")]
+    pub protocol: String,
+
+    /// number of directory scan bars to show at any given time, 0 is no limit
+    #[serde(default)]
+    pub limit_bars: usize,
+
+    /// only show unique responses based on status code and word count
+    #[serde(default)]
+    pub unique: bool,
+
+    /// Maximum size of response to read in bytes (default: 4MB to prevent OOM)
+    #[serde(default = "response_size_limit")]
+    pub response_size_limit: usize,
 }
 
 impl Default for Configuration {
@@ -375,10 +406,12 @@ impl Default for Configuration {
             resumed: false,
             stdin: false,
             json: false,
+            scan_dir_listings: false,
             verbosity: 0,
             scan_limit: 0,
             parallel: 0,
             rate_limit: 0,
+            limit_bars: 0,
             add_slash: false,
             insecure: false,
             redirects: false,
@@ -400,6 +433,8 @@ impl Default for Configuration {
             time_limit: String::new(),
             resume_from: String::new(),
             replay_proxy: String::new(),
+            request_file: String::new(),
+            protocol: request_protocol(),
             server_certs: Vec::new(),
             queries: Vec::new(),
             extensions: Vec::new(),
@@ -418,6 +453,9 @@ impl Default for Configuration {
             threads: threads(),
             wordlist: wordlist(),
             dont_collect: ignored_extensions(),
+            backup_extensions: backup_extensions(),
+            unique: false,
+            response_size_limit: response_size_limit(),
         }
     }
 }
@@ -450,6 +488,7 @@ impl Configuration {
     /// - **extensions**: `None`
     /// - **collect_extensions**: `false`
     /// - **collect_backups**: `false`
+    /// - **backup_extensions**: [`DEFAULT_BACKUP_EXTENSIONS`](constant.DEFAULT_BACKUP_EXTENSIONS.html)
     /// - **collect_words**: `false`
     /// - **dont_collect**: [`DEFAULT_IGNORED_EXTENSIONS`](constant.DEFAULT_RESPONSE_CODES.html)
     /// - **methods**: [`DEFAULT_METHOD`](constant.DEFAULT_METHOD.html)
@@ -471,12 +510,17 @@ impl Configuration {
     /// - **depth**: `4` (maximum recursion depth)
     /// - **force_recursion**: `false` (still respects recursion depth)
     /// - **scan_limit**: `0` (no limit on concurrent scans imposed)
+    /// - **limit_bars**: `0` (no limit on number of directory scan bars shown)
     /// - **parallel**: `0` (no limit on parallel scans imposed)
     /// - **rate_limit**: `0` (no limit on requests per second imposed)
     /// - **time_limit**: `None` (no limit on length of scan imposed)
     /// - **replay_proxy**: `None` (no limit on concurrent scans imposed)
     /// - **replay_codes**: [`DEFAULT_RESPONSE_CODES`](constant.DEFAULT_RESPONSE_CODES.html)
     /// - **update_app**: `false`
+    /// - **scan_dir_listings**: `false`
+    /// - **request_file**: `None`
+    /// - **protocol**: `https`
+    /// - **unique**: `false`
     ///
     /// After which, any values defined in a
     /// [ferox-config.toml](constant.DEFAULT_CONFIG_NAME.html) config file will override the
@@ -550,6 +594,18 @@ impl Configuration {
         // merge the cli options into the config file options and return the result
         Self::merge_config(&mut config, cli_config);
 
+        // if the user provided a raw request file as the target, we'll need to parse out
+        // the provided info and update the config with those values. This call needs to
+        // come after the cli/config merge so we can allow the cli options to override
+        // the raw request values (i.e. --headers "stuff: things" should override a "stuff"
+        // header from the raw request).
+        //
+        // Additionally, this call needs to come before client rebuild so that the things
+        // like user-agent can be set at the client level instead of the header level.
+        if !config.request_file.is_empty() {
+            parse_request_file(&mut config)?;
+        }
+
         // rebuild clients is the last step in either code branch
         Self::try_rebuild_clients(&mut config);
 
@@ -571,9 +627,7 @@ impl Configuration {
         //   - current directory
 
         // merge a config found at /etc/feroxbuster/ferox-config.toml
-        let config_file = PathBuf::new()
-            .join("/etc/feroxbuster")
-            .join(DEFAULT_CONFIG_NAME);
+        let config_file = Path::new("/etc/feroxbuster").join(DEFAULT_CONFIG_NAME);
         Self::parse_and_merge_config(config_file, config)?;
 
         // merge a config found at ~/.config/feroxbuster/ferox-config.toml
@@ -611,13 +665,22 @@ impl Configuration {
         update_config_with_num_type_if_present!(&mut config.depth, args, "depth", usize);
         update_config_with_num_type_if_present!(&mut config.scan_limit, args, "scan_limit", usize);
         update_config_with_num_type_if_present!(&mut config.rate_limit, args, "rate_limit", usize);
+        update_config_with_num_type_if_present!(&mut config.limit_bars, args, "limit_bars", usize);
+        update_config_with_num_type_if_present!(
+            &mut config.response_size_limit,
+            args,
+            "response_size_limit",
+            usize
+        );
         update_config_if_present!(&mut config.wordlist, args, "wordlist", String);
         update_config_if_present!(&mut config.output, args, "output", String);
         update_config_if_present!(&mut config.debug_log, args, "debug_log", String);
         update_config_if_present!(&mut config.resume_from, args, "resume_from", String);
+        update_config_if_present!(&mut config.request_file, args, "request_file", String);
+        update_config_if_present!(&mut config.protocol, args, "protocol", String);
 
         if let Ok(Some(inner)) = args.try_get_one::<String>("time_limit") {
-            config.time_limit = inner.to_owned();
+            inner.clone_into(&mut config.time_limit);
         }
 
         if let Some(arg) = args.get_many::<String>("status_codes") {
@@ -641,7 +704,7 @@ impl Configuration {
                 .collect();
         } else {
             // not passed in by the user, use whatever value is held in status_codes
-            config.replay_codes = config.status_codes.clone();
+            config.replay_codes.clone_from(&config.status_codes);
         }
 
         if let Some(arg) = args.get_many::<String>("filter_status") {
@@ -655,9 +718,27 @@ impl Configuration {
         }
 
         if let Some(arg) = args.get_many::<String>("extensions") {
-            config.extensions = arg
-                .map(|val| val.trim_start_matches('.').to_string())
-                .collect();
+            let mut extensions = Vec::<String>::new();
+            for ext in arg {
+                if let Some(stripped) = ext.strip_prefix('@') {
+                    let contents = read_to_string(stripped)
+                        .unwrap_or_else(|e| report_and_exit(&e.to_string()));
+                    let exts_from_file = contents.split('\n').filter_map(|s| {
+                        let trimmed = s.trim().trim_start_matches('.');
+
+                        if trimmed.is_empty() || trimmed.starts_with('#') {
+                            None
+                        } else {
+                            Some(trimmed.to_string())
+                        }
+                    });
+
+                    extensions.extend(exts_from_file);
+                } else {
+                    extensions.push(ext.trim().trim_start_matches('.').to_string());
+                }
+            }
+            config.extensions = extensions;
         }
 
         if let Some(arg) = args.get_many::<String>("dont_collect") {
@@ -677,11 +758,15 @@ impl Configuration {
         }
 
         if let Some(arg) = args.get_one::<String>("data") {
-            if let Some(stripped) = arg.strip_prefix('@') {
-                config.data =
-                    std::fs::read(stripped).unwrap_or_else(|e| report_and_exit(&e.to_string()));
-            } else {
-                config.data = arg.as_bytes().to_vec();
+            config.parse_data_arg(arg, None);
+            if config.methods == methods() {
+                // if the user didn't specify a method, we're going to assume they meant to use POST
+                config.methods = vec![Method::POST.as_str().to_string()];
+            } else if config.methods == [Method::POST.as_str().to_string()] {
+                preconfig_log(
+                    log::LevelFilter::Info,
+                    "-m POST already implied by --data".to_string(),
+                );
             }
         }
 
@@ -770,18 +855,22 @@ impl Configuration {
                 .collect();
         }
 
-        if came_from_cli!(args, "silent") {
+        if came_from_cli!(args, "quiet") {
+            config.quiet = true;
+            config.output_level = OutputLevel::Quiet;
+        }
+
+        if came_from_cli!(args, "silent") || (config.parallel > 0 && !config.quiet) {
             // the reason this is protected by an if statement:
             // consider a user specifying silent = true in ferox-config.toml
             // if the line below is outside of the if, we'd overwrite true with
             // false if no --silent is used on the command line
             config.silent = true;
-            config.output_level = OutputLevel::Silent;
-        }
-
-        if came_from_cli!(args, "quiet") {
-            config.quiet = true;
-            config.output_level = OutputLevel::Quiet;
+            config.output_level = if config.json {
+                OutputLevel::SilentJSON
+            } else {
+                OutputLevel::Silent
+            };
         }
 
         if came_from_cli!(args, "auto_tune")
@@ -801,6 +890,10 @@ impl Configuration {
             config.save_state = false;
         }
 
+        if came_from_cli!(args, "scan_dir_listings") || came_from_cli!(args, "thorough") {
+            config.scan_dir_listings = true;
+        }
+
         if came_from_cli!(args, "dont_filter") {
             config.dont_filter = true;
         }
@@ -814,6 +907,20 @@ impl Configuration {
             || came_from_cli!(args, "thorough")
         {
             config.collect_backups = true;
+            config.backup_extensions = backup_extensions();
+
+            if came_from_cli!(args, "collect_backups") {
+                if let Some(arg) = args.get_many::<String>("collect_backups") {
+                    let backup_exts = arg
+                        .map(|ext| ext.trim().to_string())
+                        .collect::<Vec<String>>();
+
+                    if !backup_exts.is_empty() {
+                        // have at least one cli backup, override the defaults
+                        config.backup_extensions = backup_exts;
+                    }
+                }
+            }
         }
 
         if came_from_cli!(args, "collect_words")
@@ -827,6 +934,25 @@ impl Configuration {
             // occurrences_of returns 0 if none are found; this is protected in
             // an if block for the same reason as the quiet option
             config.verbosity = args.get_count("verbosity");
+
+            // todo: starting on 2.11.0 (907-dont-skip-dir-listings), trace-level
+            //   logging started causing the following error:
+            //
+            // thread 'tokio-runtime-worker' has overflowed its stack
+            // fatal runtime error: stack overflow
+            // Aborted (core dumped)
+            //
+            // as a temporary fix, we'll disable trace logging to prevent the stack
+            // overflow until I get time to investigate the root cause
+            if config.verbosity > 3 {
+                eprintln!(
+                    "{} {}: Trace level logging is disabled; setting log level to debug",
+                    status_colorizer("WRN"),
+                    module_colorizer("Configuration::parse_cli_args"),
+                );
+
+                config.verbosity = 3;
+            }
         }
 
         if came_from_cli!(args, "no_recursion") {
@@ -853,6 +979,10 @@ impl Configuration {
             config.update_app = true;
         }
 
+        if came_from_cli!(args, "unique") {
+            config.unique = true;
+        }
+
         ////
         // organizational breakpoint; all options below alter the Client configuration
         ////
@@ -867,6 +997,48 @@ impl Configuration {
             config.proxy = String::from("http://127.0.0.1:8080");
         }
 
+        if came_from_cli!(args, "data-urlencoded") {
+            let arg = args.get_one::<String>("data-urlencoded").unwrap();
+            config.parse_data_arg(arg, Some(ContentType::UrlEncoded));
+            let default_methods = vec![Method::POST.as_str().to_string()];
+
+            if config.methods == methods() {
+                // if the user didn't specify a method, we're going to assume they meant to use POST
+                config.methods = default_methods;
+            } else if config.methods == default_methods {
+                preconfig_log(
+                    log::LevelFilter::Info,
+                    "-m POST already implied by --data-urlencoded".to_string(),
+                );
+            }
+
+            config.headers.insert(
+                String::from_str("Content-Type").unwrap(),
+                ContentType::UrlEncoded.to_header_value(),
+            );
+        }
+
+        if came_from_cli!(args, "data-json") {
+            let arg = args.get_one::<String>("data-json").unwrap();
+            config.parse_data_arg(arg, Some(ContentType::Json));
+            let default_methods = vec![Method::POST.as_str().to_string()];
+
+            if config.methods == methods() {
+                // if the user didn't specify a method, we're going to assume they meant to use POST
+                config.methods = default_methods;
+            } else if config.methods == default_methods {
+                preconfig_log(
+                    log::LevelFilter::Info,
+                    "-m POST already implied by --data-json".to_string(),
+                );
+            }
+
+            config.headers.insert(
+                String::from_str("Content-Type").unwrap(),
+                ContentType::Json.to_header_value(),
+            );
+        }
+
         if came_from_cli!(args, "burp_replay") {
             config.replay_proxy = String::from("http://127.0.0.1:8080");
         }
@@ -888,15 +1060,11 @@ impl Configuration {
 
         if let Some(headers) = args.get_many::<String>("headers") {
             for val in headers {
-                let mut split_val = val.split(':');
-
-                // explicitly take first split value as header's name
-                let name = split_val.next().unwrap().trim();
-
-                // all other items in the iterator returned by split, when combined with the
-                // original split deliminator (:), make up the header's final value
-                let value = split_val.collect::<Vec<&str>>().join(":");
-                config.headers.insert(name.to_string(), value.to_string());
+                let Ok((name, value)) = split_header(val) else {
+                    preconfig_log(log::LevelFilter::Info, format!("Invalid header: {val}"));
+                    continue;
+                };
+                config.headers.insert(name, value);
             }
         }
 
@@ -904,28 +1072,43 @@ impl Configuration {
             config.headers.insert(
                 // we know the header name is always "cookie"
                 "Cookie".to_string(),
-                // on splitting, there should be only two elements,
-                // a key and a value
                 cookies
-                    .map(|cookie| cookie.split('=').collect::<Vec<&str>>()[..].to_owned())
-                    .filter(|parts| parts.len() == 2)
-                    .map(|parts| format!("{}={}", parts[0].trim(), parts[1].trim()))
-                    // trim the spaces, join with an equals sign
+                    .flat_map(|cookie| {
+                        cookie.split(';').filter_map(|part| {
+                            // trim the spaces
+                            let trimmed = part.trim();
+                            if trimmed.is_empty() {
+                                None
+                            } else {
+                                // Find the position of the first equals sign
+                                if let Some(pos) = trimmed.find('=') {
+                                    // Split into name and value at the first equals sign
+                                    let name = &trimmed[..pos].trim();
+                                    let value = &trimmed[pos + 1..].trim();
+                                    Some(format!("{name}={value}"))
+                                } else {
+                                    // Handle the case where there's no equals sign
+                                    Some(trimmed.to_string())
+                                }
+                            }
+                        })
+                    })
                     .collect::<Vec<String>>()
-                    .join("; "), // join all the cookies with semicolons for the final header
+                    // join all the cookies with semicolons for the final header
+                    .join("; "),
             );
         }
 
         if let Some(queries) = args.get_many::<String>("queries") {
             for val in queries {
-                // same basic logic used as reading in the headers HashMap above
-                let mut split_val = val.split('=');
-
-                let name = split_val.next().unwrap().trim();
-
-                let value = split_val.collect::<Vec<&str>>().join("=");
-
-                config.queries.push((name.to_string(), value.to_string()));
+                let Ok((name, value)) = split_query(val) else {
+                    preconfig_log(
+                        log::LevelFilter::Warn,
+                        format!("Invalid query string: {val}"),
+                    );
+                    continue;
+                };
+                config.queries.push((name, value));
             }
         }
 
@@ -1012,7 +1195,7 @@ impl Configuration {
 
     /// Given a configuration file's location and an instance of `Configuration`, read in
     /// the config file if found and update the current settings with the settings found therein
-    fn parse_and_merge_config(config_file: PathBuf, mut config: &mut Self) -> Result<()> {
+    fn parse_and_merge_config(config_file: PathBuf, config: &mut Self) -> Result<()> {
         if config_file.exists() {
             // save off a string version of the path before it goes out of scope
             let conf_str = config_file.to_str().unwrap_or("").to_string();
@@ -1043,9 +1226,11 @@ impl Configuration {
             new.server_certs,
             Vec::<String>::new()
         );
+        update_if_not_default!(&mut conf.json, new.json, false);
         update_if_not_default!(&mut conf.client_cert, new.client_cert, "");
         update_if_not_default!(&mut conf.client_key, new.client_key, "");
         update_if_not_default!(&mut conf.verbosity, new.verbosity, 0);
+        update_if_not_default!(&mut conf.limit_bars, new.limit_bars, 0);
         update_if_not_default!(&mut conf.silent, new.silent, false);
         update_if_not_default!(&mut conf.quiet, new.quiet, false);
         update_if_not_default!(&mut conf.auto_bail, new.auto_bail, false);
@@ -1054,7 +1239,7 @@ impl Configuration {
         update_if_not_default!(&mut conf.collect_backups, new.collect_backups, false);
         update_if_not_default!(&mut conf.collect_words, new.collect_words, false);
         // use updated quiet/silent values to determine output level; same for requester policy
-        conf.output_level = determine_output_level(conf.quiet, conf.silent);
+        conf.output_level = determine_output_level(conf.quiet, conf.silent, conf.json);
         conf.requester_policy = determine_requester_policy(conf.auto_tune, conf.auto_bail);
         update_if_not_default!(&mut conf.output, new.output, "");
         update_if_not_default!(&mut conf.redirects, new.redirects, false);
@@ -1106,16 +1291,29 @@ impl Configuration {
             Vec::<u16>::new()
         );
         update_if_not_default!(&mut conf.dont_filter, new.dont_filter, false);
+        update_if_not_default!(&mut conf.scan_dir_listings, new.scan_dir_listings, false);
         update_if_not_default!(&mut conf.scan_limit, new.scan_limit, 0);
         update_if_not_default!(&mut conf.parallel, new.parallel, 0);
         update_if_not_default!(&mut conf.rate_limit, new.rate_limit, 0);
         update_if_not_default!(&mut conf.replay_proxy, new.replay_proxy, "");
         update_if_not_default!(&mut conf.debug_log, new.debug_log, "");
         update_if_not_default!(&mut conf.resume_from, new.resume_from, "");
-        update_if_not_default!(&mut conf.json, new.json, false);
+        update_if_not_default!(&mut conf.request_file, new.request_file, "");
+        update_if_not_default!(&mut conf.protocol, new.protocol, request_protocol());
+        update_if_not_default!(&mut conf.unique, new.unique, false);
+        update_if_not_default!(
+            &mut conf.response_size_limit,
+            new.response_size_limit,
+            response_size_limit()
+        );
 
         update_if_not_default!(&mut conf.timeout, new.timeout, timeout());
         update_if_not_default!(&mut conf.user_agent, new.user_agent, user_agent());
+        update_if_not_default!(
+            &mut conf.backup_extensions,
+            new.backup_extensions,
+            backup_extensions()
+        );
         update_if_not_default!(&mut conf.random_agent, new.random_agent, false);
         update_if_not_default!(&mut conf.threads, new.threads, threads());
         update_if_not_default!(&mut conf.depth, new.depth, depth());
@@ -1149,6 +1347,37 @@ impl Configuration {
 
         Ok(config)
     }
+
+    /// Reads payload body from STDIN or file system depending on '@' and
+    ///
+    /// sets config.data according to the body's content type
+    fn parse_data_arg(&mut self, arg: &str, content_type: Option<ContentType>) {
+        let mut payload: String;
+
+        if let Some(stripped) = arg.strip_prefix('@') {
+            payload = std::fs::read_to_string(stripped)
+                .unwrap_or_else(|e| report_and_exit(&e.to_string()))
+        } else {
+            payload = arg.to_string();
+        }
+
+        match content_type {
+            Some(content_type) => match content_type {
+                ContentType::Json => {
+                    // because feroxbuster is a fuzzer, we do not minify or validate
+                    // the json payload with serde, for ill-formed JSON might be used
+                    self.data = payload.as_bytes().to_vec()
+                }
+                ContentType::UrlEncoded => {
+                    payload = payload.replace("\r\n", "&").replace("\n", "&");
+                    let encoded: String =
+                        form_urlencoded::byte_serialize(payload.as_bytes()).collect();
+                    self.data = encoded.as_bytes().to_vec();
+                }
+            },
+            None => self.data = payload.as_bytes().to_vec(),
+        }
+    }
 }
 
 /// Implementation of FeroxMessage
