@@ -3,6 +3,7 @@ use crate::filters::{
     FeroxFilters, LinesFilter, RegexFilter, SimilarityFilter, SizeFilter, StatusCodeFilter,
     WordsFilter,
 };
+use crate::sync::DynamicSemaphore;
 use crate::{
     config::{Configuration, OutputLevel},
     event_handlers::Handles,
@@ -10,11 +11,12 @@ use crate::{
     scanner::RESPONSES,
     statistics::Stats,
     traits::FeroxSerialize,
-    SLEEP_DURATION, VERSION,
+    NEAR_DUPLICATE_DISTANCE, SLEEP_DURATION, VERSION,
 };
 use indicatif::ProgressBar;
 use predicates::prelude::*;
 use regex::Regex;
+use std::sync::atomic::AtomicBool;
 use std::sync::{atomic::Ordering, Arc};
 use std::thread::sleep;
 use std::time::Instant;
@@ -47,7 +49,8 @@ async fn scanner_pause_scan_with_finished_spinner() {
         PAUSE_SCAN.store(false, Ordering::Relaxed);
     });
 
-    urls.pause(false, handles).await;
+    urls.pause(false, handles, Arc::new(DynamicSemaphore::new(100)))
+        .await;
 
     assert!(now.elapsed() > expected);
 }
@@ -57,7 +60,12 @@ async fn scanner_pause_scan_with_finished_spinner() {
 fn add_url_to_list_of_scanned_urls_with_unknown_url() {
     let urls = FeroxScans::default();
     let url = "http://unknown_url";
-    let (result, _scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest);
+    let (result, _scan) = urls.add_scan(
+        url,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        Arc::new(Handles::for_testing(None, None).0),
+    );
     assert!(result);
 }
 
@@ -75,11 +83,18 @@ fn add_url_to_list_of_scanned_urls_with_known_url() {
         pb.length().unwrap(),
         OutputLevel::Default,
         Some(pb),
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
 
     assert!(urls.insert(scan));
 
-    let (result, _scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest);
+    let (result, _scan) = urls.add_scan(
+        url,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        Arc::new(Handles::for_testing(None, None).0),
+    );
 
     assert!(!result);
 }
@@ -97,6 +112,8 @@ fn stop_progress_bar_stops_bar() {
         pb.length().unwrap(),
         OutputLevel::Default,
         Some(pb),
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
 
     assert!(!scan
@@ -107,7 +124,7 @@ fn stop_progress_bar_stops_bar() {
         .unwrap()
         .is_finished());
 
-    scan.stop_progress_bar();
+    scan.stop_progress_bar(0);
 
     assert!(scan
         .progress_bar
@@ -124,18 +141,25 @@ fn add_url_to_list_of_scanned_urls_with_known_url_without_slash() {
     let urls = FeroxScans::default();
     let url = "http://unknown_url";
 
-    let scan = FeroxScan::new(
+    let scan: Arc<FeroxScan> = FeroxScan::new(
         url,
         ScanType::File,
         ScanOrder::Latest,
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
 
     assert!(urls.insert(scan));
 
-    let (result, _scan) = urls.add_scan(url, ScanType::File, ScanOrder::Latest);
+    let (result, _scan) = urls.add_scan(
+        url,
+        ScanType::File,
+        ScanOrder::Latest,
+        Arc::new(Handles::for_testing(None, None).0),
+    );
 
     assert!(!result);
 }
@@ -155,6 +179,8 @@ async fn call_display_scans() {
         pb.length().unwrap(),
         OutputLevel::Default,
         Some(pb),
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
     let scan_two = FeroxScan::new(
         url_two,
@@ -163,9 +189,11 @@ async fn call_display_scans() {
         pb_two.length().unwrap(),
         OutputLevel::Default,
         Some(pb_two),
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
 
-    scan_two.finish().unwrap(); // one complete, one incomplete
+    scan_two.finish(0).unwrap(); // one complete, one incomplete
     scan_two
         .set_task(tokio::spawn(async move {
             sleep(Duration::from_millis(SLEEP_DURATION));
@@ -190,6 +218,8 @@ fn partial_eq_compares_the_id_field() {
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
     let scan_two = FeroxScan::new(
         url,
@@ -198,10 +228,13 @@ fn partial_eq_compares_the_id_field() {
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
 
     assert!(!scan.eq(&scan_two));
 
+    #[allow(clippy::redundant_clone)]
     let scan_two = scan.clone();
 
     assert!(scan.eq(&scan_two));
@@ -279,6 +312,8 @@ fn ferox_scan_serialize() {
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
     let fs_json = format!(
         r#"{{"id":"{}","url":"https://spiritanimal.com","normalized_url":"https://spiritanimal.com/","scan_type":"Directory","status":"NotStarted","num_requests":0,"requests_made_so_far":0}}"#,
@@ -297,6 +332,8 @@ fn ferox_scans_serialize() {
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
     let ferox_scans = FeroxScans::default();
     let ferox_scans_json = format!(
@@ -313,7 +350,7 @@ fn ferox_scans_serialize() {
 #[test]
 /// given a FeroxResponses, test that it serializes into the proper JSON entry
 fn ferox_responses_serialize() {
-    let json_response = r#"{"type":"response","url":"https://nerdcore.com/css","original_url":"https://nerdcore.com","path":"/css","wildcard":true,"status":301,"method":"GET","content_length":173,"line_count":10,"word_count":16,"headers":{"server":"nginx/1.16.1"},"extension":""}"#;
+    let json_response = r#"{"type":"response","url":"https://nerdcore.com/css","original_url":"https://nerdcore.com","path":"/css","wildcard":true,"status":301,"method":"GET","content_length":173,"line_count":10,"word_count":16,"headers":{"server":"nginx/1.16.1"},"extension":"","truncated":false,"timestamp":1711796681.3455093}"#;
     let response: FeroxResponse = serde_json::from_str(json_response).unwrap();
 
     let responses = FeroxResponses::default();
@@ -331,7 +368,7 @@ fn ferox_responses_serialize() {
 /// given a FeroxResponse, test that it serializes into the proper JSON entry
 fn ferox_response_serialize_and_deserialize() {
     // deserialize
-    let json_response = r#"{"type":"response","url":"https://nerdcore.com/css","original_url":"https://nerdcore.com","path":"/css","wildcard":true,"status":301,"method":"GET","content_length":173,"line_count":10,"word_count":16,"headers":{"server":"nginx/1.16.1"},"extension":""}"#;
+    let json_response = r#"{"type":"response","url":"https://nerdcore.com/css","original_url":"https://nerdcore.com","path":"/css","wildcard":true,"status":301,"method":"GET","content_length":173,"line_count":10,"word_count":16,"headers":{"server":"nginx/1.16.1"},"extension":"","truncated":false,"timestamp":1711796681.3455093}"#;
     let response: FeroxResponse = serde_json::from_str(json_response).unwrap();
 
     assert_eq!(response.url().as_str(), "https://nerdcore.com/css");
@@ -342,6 +379,8 @@ fn ferox_response_serialize_and_deserialize() {
     assert_eq!(response.line_count(), 10);
     assert_eq!(response.word_count(), 16);
     assert_eq!(response.headers().get("server").unwrap(), "nginx/1.16.1");
+    assert_eq!(response.timestamp(), 1711796681.3455093);
+    assert!(!response.truncated());
 
     // serialize, however, this can fail when headers are out of order
     let new_json = serde_json::to_string(&response).unwrap();
@@ -358,6 +397,8 @@ fn feroxstates_feroxserialize_implementation() {
         0,
         OutputLevel::Default,
         None,
+        true,
+        Arc::new(Handles::for_testing(None, None).0),
     );
     let ferox_scans = FeroxScans::default();
     let saved_id = ferox_scan.id.clone();
@@ -405,6 +446,7 @@ fn feroxstates_feroxserialize_implementation() {
         .push(Box::new(SimilarityFilter {
             hash: 1,
             original_url: "http://localhost:12345/".to_string(),
+            cutoff: NEAR_DUPLICATE_DISTANCE,
         }))
         .unwrap();
 
@@ -499,13 +541,18 @@ fn feroxstates_feroxserialize_implementation() {
         r#""method":"GET""#,
         r#""content_length":173"#,
         r#""line_count":10"#,
+        r#""limit_bars":0"#,
         r#""word_count":16"#,
         r#""headers""#,
         r#""server":"nginx/1.16.1"#,
         r#""collect_extensions":true"#,
         r#""collect_backups":false"#,
         r#""collect_words":false"#,
-        r#""filters":[{"filter_code":100},{"word_count":200},{"content_length":300},{"line_count":400},{"compiled":".*","raw_string":".*"},{"hash":1,"original_url":"http://localhost:12345/"}]"#,
+        r#""scan_dir_listings":false"#,
+        r#""protocol":"https""#,
+        r#""unique":false"#,
+        r#""response_size_limit":4194304"#,
+        r#""filters":[{"filter_code":100},{"word_count":200},{"content_length":300},{"line_count":400},{"compiled":".*","raw_string":".*"},{"hash":1,"original_url":"http://localhost:12345/","cutoff":3}]"#,
         r#""collected_extensions":["php"]"#,
         r#""dont_collect":["tif","tiff","ico","cur","bmp","webp","svg","png","jpg","jpeg","jfif","gif","avif","apng","pjpeg","pjp","mov","wav","mpg","mpeg","mp3","mp4","m4a","m4p","m4v","ogg","webm","ogv","oga","flac","aac","3gp","css","zip","xls","xml","gz","tgz"]"#,
     ]
@@ -565,8 +612,10 @@ fn feroxscan_display() {
         normalized_url: String::from("http://localhost/"),
         scan_order: ScanOrder::Latest,
         scan_type: Default::default(),
+        handles: Some(Arc::new(Handles::for_testing(None, None).0)),
         num_requests: 0,
         requests_made_so_far: 0,
+        visible: AtomicBool::new(true),
         start_time: Instant::now(),
         output_level: OutputLevel::Default,
         status_403s: Default::default(),
@@ -615,6 +664,7 @@ async fn ferox_scan_abort() {
         requests_made_so_far: 0,
         start_time: Instant::now(),
         output_level: OutputLevel::Default,
+        visible: AtomicBool::new(true),
         status_403s: Default::default(),
         status_429s: Default::default(),
         status: std::sync::Mutex::new(ScanStatus::Running),
@@ -623,9 +673,10 @@ async fn ferox_scan_abort() {
         }))),
         progress_bar: std::sync::Mutex::new(None),
         errors: Default::default(),
+        handles: Some(Arc::new(Handles::for_testing(None, None).0)),
     };
 
-    scan.abort().await.unwrap();
+    scan.abort(0).await.unwrap();
 
     assert!(matches!(
         *scan.status.lock().unwrap(),
@@ -658,7 +709,7 @@ fn menu_get_command_input_from_user_returns_cancel() {
     let menu = Menu::new();
 
     for (idx, cmd) in ["cancel", "Cancel", "c", "C"].iter().enumerate() {
-        let force = idx % 2 == 0;
+        let force = idx.is_multiple_of(2);
 
         let full_cmd = if force {
             format!("{cmd} -f {idx}\n")
@@ -716,15 +767,26 @@ fn split_to_nums_is_correct() {
 #[test]
 /// given a deep url, find the correct scan
 fn get_base_scan_by_url_finds_correct_scan() {
+    let handles = Arc::new(Handles::for_testing(None, None).0);
     let urls = FeroxScans::default();
     let url = "http://localhost";
     let url1 = "http://localhost/stuff";
     let url2 = "http://shlocalhost/stuff/things";
     let url3 = "http://shlocalhost/stuff/things/mostuff";
-    let (_, scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest);
-    let (_, scan1) = urls.add_scan(url1, ScanType::Directory, ScanOrder::Latest);
-    let (_, scan2) = urls.add_scan(url2, ScanType::Directory, ScanOrder::Latest);
-    let (_, scan3) = urls.add_scan(url3, ScanType::Directory, ScanOrder::Latest);
+    let (_, scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest, handles.clone());
+    let (_, scan1) = urls.add_scan(
+        url1,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        handles.clone(),
+    );
+    let (_, scan2) = urls.add_scan(
+        url2,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        handles.clone(),
+    );
+    let (_, scan3) = urls.add_scan(url3, ScanType::Directory, ScanOrder::Latest, handles);
 
     assert_eq!(
         urls.get_base_scan_by_url("http://localhost/things.php")
@@ -757,7 +819,12 @@ fn get_base_scan_by_url_finds_correct_scan() {
 fn get_base_scan_by_url_finds_correct_scan_without_trailing_slash() {
     let urls = FeroxScans::default();
     let url = "http://localhost";
-    let (_, scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest);
+    let (_, scan) = urls.add_scan(
+        url,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        Arc::new(Handles::for_testing(None, None).0),
+    );
     assert_eq!(
         urls.get_base_scan_by_url("http://localhost/BKPMiherrortBPKcw")
             .unwrap()
@@ -771,7 +838,12 @@ fn get_base_scan_by_url_finds_correct_scan_without_trailing_slash() {
 fn get_base_scan_by_url_finds_correct_scan_with_trailing_slash() {
     let urls = FeroxScans::default();
     let url = "http://127.0.0.1:41971/";
-    let (_, scan) = urls.add_scan(url, ScanType::Directory, ScanOrder::Latest);
+    let (_, scan) = urls.add_scan(
+        url,
+        ScanType::Directory,
+        ScanOrder::Latest,
+        Arc::new(Handles::for_testing(None, None).0),
+    );
     assert_eq!(
         urls.get_base_scan_by_url("http://127.0.0.1:41971/BKPMiherrortBPKcw")
             .unwrap()
