@@ -25,13 +25,14 @@ use crate::{
         Handles,
     },
     extractor::{ExtractionTarget, ExtractorBuilder},
+    filters::SimilarityFilter,
     nlp::{Document, TfIdf},
     response::FeroxResponse,
     scan_manager::{FeroxScan, ScanStatus},
     statistics::{StatError::Other, StatField::TotalExpected},
     url::FeroxUrl,
     utils::{logged_request, send_try_recursion_command, should_deny_url},
-    HIGH_ERROR_RATIO,
+    HIGH_ERROR_RATIO, UNIQUE_DISTANCE,
 };
 
 use super::{policy_data::PolicyData, FeroxScanner, PolicyTrigger};
@@ -311,7 +312,7 @@ impl Requester {
             // minimum number of requests entering this block
             self.ferox_scan
                 .set_status(ScanStatus::Cancelled)
-                .unwrap_or_else(|e| log::warn!("Could not set scan status: {}", e));
+                .unwrap_or_else(|e| log::warn!("Could not set scan status: {e}"));
 
             let scans = self.handles.ferox_scans()?;
             let active_bars = scans.number_of_bars();
@@ -320,7 +321,7 @@ impl Requester {
             self.ferox_scan
                 .abort(active_bars)
                 .await
-                .unwrap_or_else(|e| log::warn!("Could not bail on scan: {}", e));
+                .unwrap_or_else(|e| log::warn!("Could not bail on scan: {e}"));
 
             // figure out how many requests are skipped as a result
             let pb = self.ferox_scan.progress_bar();
@@ -339,7 +340,7 @@ impl Requester {
             self.handles
                 .stats
                 .send(SubtractFromUsizeField(TotalExpected, num_skipped))
-                .unwrap_or_else(|e| log::warn!("Could not update overall scan bar: {}", e));
+                .unwrap_or_else(|e| log::warn!("Could not update overall scan bar: {e}"));
         }
 
         Ok(())
@@ -349,7 +350,7 @@ impl Requester {
     ///
     /// Attempts recursion when appropriate and sends Responses to the output handler for processing
     pub async fn request(&self, word: &str) -> Result<()> {
-        log::trace!("enter: request({})", word);
+        log::trace!("enter: request({word})");
 
         let collected = self.handles.collected_extensions();
 
@@ -371,7 +372,7 @@ impl Requester {
                 if should_limit {
                     // found a rate limiter, limit that junk!
                     if let Err(e) = self.limit().await {
-                        log::warn!("Could not rate limit scan: {}", e);
+                        log::warn!("Could not rate limit scan: {e}");
                         self.handles.stats.send(AddError(Other)).unwrap_or_default();
                     }
                 }
@@ -420,6 +421,7 @@ impl Requester {
                     &self.target_url,
                     method,
                     self.handles.config.output_level,
+                    self.handles.config.response_size_limit,
                 )
                 .await;
 
@@ -445,6 +447,12 @@ impl Requester {
                     continue;
                 }
 
+                if self.handles.config.unique {
+                    let mut unique_filter = SimilarityFilter::from(&ferox_response);
+                    unique_filter.cutoff = UNIQUE_DISTANCE;
+                    self.handles.filters.data.push(Box::new(unique_filter))?;
+                }
+
                 if !self.handles.config.no_recursion && self.handles.config.force_recursion {
                     // in this branch, we're saying that both recursion AND force recursion
                     // are turned on. It comes after should_filter_response, so those cases
@@ -480,8 +488,9 @@ impl Requester {
                     if let Ok(mut guard) = TF_IDF.write() {
                         if let Some(doc) = Document::from_html(ferox_response.text()) {
                             guard.add_document(doc);
-                            if guard.num_documents() % 12 == 0
-                                || (guard.num_documents() < 5 && guard.num_documents() % 2 == 0)
+                            if guard.num_documents().is_multiple_of(12)
+                                || (guard.num_documents() < 5
+                                    && guard.num_documents().is_multiple_of(2))
                             {
                                 guard.calculate_tf_idf_scores();
                             }
@@ -528,7 +537,7 @@ impl Requester {
 
                 // everything else should be reported
                 if let Err(e) = ferox_response.send_report(self.handles.output.tx.clone()) {
-                    log::warn!("Could not send FeroxResponse to output handler: {}", e);
+                    log::warn!("Could not send FeroxResponse to output handler: {e}");
                 }
             }
         }
